@@ -1,17 +1,19 @@
+from __future__ import annotations
+
 import asyncio
 from typing import Any, Awaitable
 
-from quart.globals import _app_ctx_stack, _request_ctx_stack, _websocket_ctx_stack
+from quart.globals import _cv_app, _cv_request, _cv_websocket
 
 
 def sync_with_context(future: Awaitable) -> Any:
-    context = None
-    if _request_ctx_stack.top is not None:
-        context = _request_ctx_stack.top.copy()
-    elif _websocket_ctx_stack.top is not None:
-        context = _websocket_ctx_stack.top.copy()
-    elif _app_ctx_stack.top is not None:
-        context = _app_ctx_stack.top.copy()
+    context: Any = None
+    if _cv_request.get(None) is not None:
+        context = _cv_request.get().copy()
+    elif _cv_websocket.get(None) is not None:
+        context = _cv_websocket.get().copy()
+    elif _cv_app.get(None) is not None:
+        context = _cv_app.get().copy()
 
     async def context_wrapper() -> Any:
         if context is not None:
