@@ -22,13 +22,14 @@ class QuartRule(Rule):
         websocket: bool = False,
         provide_automatic_options: bool = False,
     ) -> None:
-        super().__init__(  # type: ignore
+        super().__init__(
             string,
             defaults=defaults,
             subdomain=subdomain,
             methods=methods,
             endpoint=endpoint,
             strict_slashes=strict_slashes,
+            merge_slashes=merge_slashes,
             host=host,
             websocket=websocket,
         )
@@ -37,7 +38,7 @@ class QuartRule(Rule):
 
 class QuartMap(Map):
     def bind_to_request(
-        self, request: BaseRequestWebsocket, subdomain: Optional[str], server_name: Optional[str],
+        self, request: BaseRequestWebsocket, subdomain: Optional[str], server_name: Optional[str]
     ) -> MapAdapter:
         host: str
         if server_name is None:
@@ -48,7 +49,7 @@ class QuartMap(Map):
         host = _normalise_host(request.scheme, host)
 
         if subdomain is None and not self.host_matching:
-            request_host_parts = _normalise_host(request.scheme, request.host).split(".")
+            request_host_parts = _normalise_host(request.scheme, request.host.lower()).split(".")
             config_host_parts = host.split(".")
             offset = -len(config_host_parts)
 
@@ -62,14 +63,22 @@ class QuartMap(Map):
             else:
                 subdomain = ".".join(filter(None, request_host_parts[:offset]))
 
+        if request.root_path:
+            try:
+                path = request.path.split(request.root_path, 1)[1]
+            except IndexError:
+                path = " "  # Invalid in paths, hence will result in 404
+        else:
+            path = request.path
+
         return super().bind(
             host,
             request.root_path,
             subdomain,
             request.scheme,
             request.method,
-            request.path,
-            request.query_string,
+            path,
+            request.query_string.decode(),
         )
 
 
