@@ -1,11 +1,8 @@
 from __future__ import annotations
 
 import os
-import pkgutil
-import sys
 from collections import defaultdict
 from functools import wraps
-from json import JSONDecoder, JSONEncoder
 from pathlib import Path
 from typing import (
     Any,
@@ -30,8 +27,8 @@ from werkzeug.exceptions import default_exceptions, HTTPException
 
 from .cli import AppGroup
 from .globals import current_app
-from .helpers import send_from_directory
-from .templating import _default_template_context_processor
+from .helpers import get_root_path, send_from_directory
+from .templating import _default_template_ctx_processor
 from .typing import (
     AfterRequestCallable,
     AfterWebsocketCallable,
@@ -40,10 +37,12 @@ from .typing import (
     BeforeWebsocketCallable,
     ErrorHandlerCallable,
     FilePath,
+    RouteCallable,
     TeardownCallable,
     TemplateContextProcessorCallable,
     URLDefaultCallable,
     URLValuePreprocessorCallable,
+    WebsocketCallable,
 )
 from .utils import file_path_to_path
 
@@ -52,33 +51,33 @@ if TYPE_CHECKING:
 
 
 F = TypeVar("F", bound=Callable)
+T_after_request = TypeVar("T_after_request", bound=AfterRequestCallable)
+T_after_websocket = TypeVar("T_after_websocket", bound=AfterWebsocketCallable)
+T_before_request = TypeVar("T_before_request", bound=BeforeRequestCallable)
+T_before_websocket = TypeVar("T_before_websocket", bound=BeforeWebsocketCallable)
+T_error_handler = TypeVar("T_error_handler", bound=ErrorHandlerCallable)
+T_teardown = TypeVar("T_teardown", bound=TeardownCallable)
+T_template_context_processor = TypeVar(
+    "T_template_context_processor", bound=TemplateContextProcessorCallable
+)
+T_url_defaults = TypeVar("T_url_defaults", bound=URLDefaultCallable)
+T_url_value_preprocessor = TypeVar("T_url_value_preprocessor", bound=URLValuePreprocessorCallable)
+T_route = TypeVar("T_route", bound=RouteCallable)
+T_websocket = TypeVar("T_websocket", bound=WebsocketCallable)
 
 
 def setupmethod(func: F) -> F:
     @wraps(func)
     def wrapper(self: "Scaffold", *args: Any, **kwargs: Any) -> Any:
-        if self._is_setup_finished():
-            raise AssertionError(
-                "The first request has already been handled by the app, "
-                "calling this function now has no affect."
-            )
+        self._check_setup_finished(func.__name__)
         return func(self, *args, **kwargs)
 
     return cast(F, wrapper)
 
 
 class Scaffold:
-    """Base class for Quart and Blueprint classes.
+    """Base class for Quart and Blueprint classes."""
 
-    Attributes:
-        json_decoder: The decoder to use for JSON data, if not set
-            will default to the app's decoder.
-        json_encoder: The encoder to use for JSON data, if not set
-            will default to the app's encoder.
-    """
-
-    json_decoder: Optional[Type[JSONDecoder]] = None
-    json_encoder: Optional[Type[JSONEncoder]] = None
     name: str
 
     def __init__(
@@ -92,7 +91,10 @@ class Scaffold:
         self.import_name = import_name
         self.template_folder = Path(template_folder) if template_folder is not None else None
 
-        self.root_path = _find_root_path(import_name, root_path)
+        if root_path is None:
+            self.root_path = Path(get_root_path(import_name))
+        else:
+            self.root_path = Path(root_path)
 
         self._static_folder: Optional[Path] = None
         self._static_url_path: Optional[str] = None
@@ -148,7 +150,7 @@ class Scaffold:
         # app).
         self.template_context_processors: Dict[
             AppOrBlueprintKey, List[TemplateContextProcessorCallable]
-        ] = defaultdict(list, {None: [_default_template_context_processor]})
+        ] = defaultdict(list, {None: [_default_template_ctx_processor]})
 
         # View functions keyed by endpoint.
         self.view_functions: Dict[str, Callable] = {}
@@ -235,29 +237,34 @@ class Scaffold:
             raise ValueError("Files can only be opened for reading")
         return async_open(self.root_path / file_path_to_path(path), mode)  # type: ignore
 
-    def _method_route(self, method: str, rule: str, options: dict) -> Callable:
+    def _method_route(self, method: str, rule: str, options: dict) -> Callable[[T_route], T_route]:
         if "methods" in options:
             raise TypeError("Methods cannot be supplied, use the 'route' decorator instead.")
 
         return self.route(rule, methods=[method], **options)
 
-    def get(self, rule: str, **options: Any) -> Callable:
+    @setupmethod
+    def get(self, rule: str, **options: Any) -> Callable[[T_route], T_route]:
         """Syntactic sugar for :meth:`route` with ``methods=["GET"]``."""
         return self._method_route("GET", rule, options)
 
-    def post(self, rule: str, **options: Any) -> Callable:
+    @setupmethod
+    def post(self, rule: str, **options: Any) -> Callable[[T_route], T_route]:
         """Syntactic sugar for :meth:`route` with ``methods=["POST"]``."""
         return self._method_route("POST", rule, options)
 
-    def put(self, rule: str, **options: Any) -> Callable:
+    @setupmethod
+    def put(self, rule: str, **options: Any) -> Callable[[T_route], T_route]:
         """Syntactic sugar for :meth:`route` with ``methods=["PUT"]``."""
         return self._method_route("PUT", rule, options)
 
-    def delete(self, rule: str, **options: Any) -> Callable:
+    @setupmethod
+    def delete(self, rule: str, **options: Any) -> Callable[[T_route], T_route]:
         """Syntactic sugar for :meth:`route` with ``methods=["DELETE"]``."""
         return self._method_route("DELETE", rule, options)
 
-    def patch(self, rule: str, **options: Any) -> Callable:
+    @setupmethod
+    def patch(self, rule: str, **options: Any) -> Callable[[T_route], T_route]:
         """Syntactic sugar for :meth:`route` with ``methods=["PATCH"]``."""
         return self._method_route("PATCH", rule, options)
 
@@ -273,7 +280,7 @@ class Scaffold:
         *,
         provide_automatic_options: Optional[bool] = None,
         strict_slashes: Optional[bool] = None,
-    ) -> Callable:
+    ) -> Callable[[T_route], T_route]:
         """Add a HTTP request handling route.
 
         This is designed to be used as a decorator, if used to
@@ -312,7 +319,7 @@ class Scaffold:
                 path. Will redirect a leaf (no slash) to a branch (with slash).
         """
 
-        def decorator(func: Callable) -> Callable:
+        def decorator(func: T_route) -> T_route:
             self.add_url_rule(
                 rule,
                 endpoint,
@@ -333,7 +340,7 @@ class Scaffold:
         self,
         rule: str,
         endpoint: Optional[str] = None,
-        view_func: Optional[Callable] = None,
+        view_func: Optional[RouteCallable] = None,
         provide_automatic_options: Optional[bool] = None,
         *,
         methods: Optional[Iterable[str]] = None,
@@ -395,7 +402,7 @@ class Scaffold:
         subdomain: Optional[str] = None,
         *,
         strict_slashes: Optional[bool] = None,
-    ) -> Callable:
+    ) -> Callable[[T_websocket], T_websocket]:
         """Add a websocket to the application.
 
         This is designed to be used as a decorator, if used to
@@ -431,7 +438,7 @@ class Scaffold:
                 path. Will redirect a leaf (no slash) to a branch (with slash).
         """
 
-        def decorator(func: Callable) -> Callable:
+        def decorator(func: T_websocket) -> T_websocket:
             self.add_websocket(
                 rule,
                 endpoint,
@@ -449,7 +456,7 @@ class Scaffold:
         self,
         rule: str,
         endpoint: Optional[str] = None,
-        view_func: Optional[Callable] = None,
+        view_func: Optional[WebsocketCallable] = None,
         defaults: Optional[dict] = None,
         host: Optional[str] = None,
         subdomain: Optional[str] = None,
@@ -504,7 +511,7 @@ class Scaffold:
         )
 
     @setupmethod
-    def endpoint(self, endpoint: str) -> Callable:
+    def endpoint(self, endpoint: str) -> Callable[[F], F]:
         """Register a function as an endpoint.
 
         This is designed to be used as a decorator, if used to
@@ -522,7 +529,7 @@ class Scaffold:
             endpoint: The endpoint name to use.
         """
 
-        def decorator(func: Callable) -> Callable:
+        def decorator(func: F) -> F:
             self.view_functions[endpoint] = func
             return func
 
@@ -531,8 +538,8 @@ class Scaffold:
     @setupmethod
     def before_request(
         self,
-        func: BeforeRequestCallable,
-    ) -> BeforeRequestCallable:
+        func: T_before_request,
+    ) -> T_before_request:
         """Add a before request function.
 
         This is designed to be used as a decorator, if used to
@@ -555,8 +562,8 @@ class Scaffold:
     @setupmethod
     def after_request(
         self,
-        func: AfterRequestCallable,
-    ) -> AfterRequestCallable:
+        func: T_after_request,
+    ) -> T_after_request:
         """Add an after request function.
 
         This is designed to be used as a decorator, if used to
@@ -579,8 +586,8 @@ class Scaffold:
     @setupmethod
     def before_websocket(
         self,
-        func: BeforeWebsocketCallable,
-    ) -> BeforeWebsocketCallable:
+        func: T_before_websocket,
+    ) -> T_before_websocket:
         """Add a before websocket function.
 
         This is designed to be used as a decorator, if used to
@@ -603,8 +610,8 @@ class Scaffold:
     @setupmethod
     def after_websocket(
         self,
-        func: AfterWebsocketCallable,
-    ) -> AfterWebsocketCallable:
+        func: T_after_websocket,
+    ) -> T_after_websocket:
         """Add an after websocket function.
 
         This is designed to be used as a decorator, if used to
@@ -627,8 +634,8 @@ class Scaffold:
     @setupmethod
     def teardown_request(
         self,
-        func: TeardownCallable,
-    ) -> TeardownCallable:
+        func: T_teardown,
+    ) -> T_teardown:
         """Add a teardown request function.
 
         This is designed to be used as a decorator, if used to
@@ -651,8 +658,8 @@ class Scaffold:
     @setupmethod
     def teardown_websocket(
         self,
-        func: TeardownCallable,
-    ) -> TeardownCallable:
+        func: T_teardown,
+    ) -> T_teardown:
         """Add a teardown websocket function.
 
         This is designed to be used as a decorator, if used to
@@ -676,8 +683,8 @@ class Scaffold:
     @setupmethod
     def context_processor(
         self,
-        func: TemplateContextProcessorCallable,
-    ) -> TemplateContextProcessorCallable:
+        func: T_template_context_processor,
+    ) -> T_template_context_processor:
         """Add a template context processor.
 
         This is designed to be used as a decorator, if used to
@@ -697,8 +704,9 @@ class Scaffold:
 
     @setupmethod
     def url_value_preprocessor(
-        self, func: URLValuePreprocessorCallable
-    ) -> URLValuePreprocessorCallable:
+        self,
+        func: T_url_value_preprocessor,
+    ) -> T_url_value_preprocessor:
         """Add a url value preprocessor.
 
         This is designed to be used as a decorator. An example usage,
@@ -713,7 +721,7 @@ class Scaffold:
         return func
 
     @setupmethod
-    def url_defaults(self, func: URLDefaultCallable) -> URLDefaultCallable:
+    def url_defaults(self, func: T_url_defaults) -> T_url_defaults:
         """Add a url default preprocessor.
 
         This is designed to be used as a decorator. An example usage,
@@ -730,7 +738,7 @@ class Scaffold:
     @setupmethod
     def errorhandler(
         self, error: Union[Type[Exception], int]
-    ) -> Callable[[ErrorHandlerCallable], ErrorHandlerCallable]:
+    ) -> Callable[[T_error_handler], T_error_handler]:
         """Register a function as an error handler.
 
         This is designed to be used as a decorator. An example usage,
@@ -745,7 +753,7 @@ class Scaffold:
             error: The error code or Exception to handle.
         """
 
-        def decorator(func: ErrorHandlerCallable) -> ErrorHandlerCallable:
+        def decorator(func: T_error_handler) -> T_error_handler:
             self.register_error_handler(error, func)
             return func
 
@@ -803,26 +811,10 @@ class Scaffold:
         else:
             return error_type, None
 
-    def _is_setup_finished(self) -> bool:
+    def _check_setup_finished(self, f_name: str) -> None:
         raise NotImplementedError()
 
 
 def _endpoint_from_view_func(view_func: Callable) -> str:
     assert view_func is not None
     return view_func.__name__
-
-
-def _find_root_path(import_name: str, root_path: Optional[str] = None) -> Path:
-    if root_path is not None:
-        return Path(root_path)
-    else:
-        module = sys.modules.get(import_name)
-        if module is not None and hasattr(module, "__file__"):
-            file_path = module.__file__
-        else:
-            loader = pkgutil.get_loader(import_name)
-            if loader is None or import_name == "__main__":
-                return Path.cwd()
-            else:
-                file_path = loader.get_filename(import_name)  # type: ignore
-        return Path(file_path).resolve().parent
